class MongoDbContext(object):

    def __init__(self, settings: dict):
        self.id = settings['id']
        self.host_name = settings['hostName']
        self.port_number = settings['portNumber']
        self.database_name = settings['databaseName']
        self.username = settings['username']
        self.password = settings['password']

    def get_connection_string(self) -> str:
        user_info_parts = []
        if self.username:
            user_info_parts.append(self.username)
        if self.password:
            user_info_parts.append(self.password)
        user_info = str.join(':', user_info_parts)
        return f'mongodb://{user_info}{"@" if user_info else ""}{self.host_name}:{self.port_number}'



import json
import os
import pathlib
from datetime import datetime
from shutil import copyfile

import click
from click import Context

from elementalcms import ElementalContext, __version__
from elementalcms.core import FlaskContext, MongoDbContext

from elementalcms.management import GlobalDeps, Pages, Snippets, Static, Media
from elementalcms.persistence import MongoDbConnectionManager
from elementalcms.services.sessions import CreateExpirationIndex


@click.group()
@click.pass_context
def cli(ctx: Context):
    """Elemental CMS management CLI"""
    ctx.ensure_object(dict)

    if not os.path.exists('settings'):
        click.echo('Settings folder do not exist, you need to create a settings folder and the prod.json '
                   'settings file to start using the CLI.')
        exit(1)

    if not os.path.exists('settings/prod.json'):
        click.echo('Settings file do not exist. Please create a production settings file in order to be able to '
                   'use the CLI.')
        exit(1)

    with open(f'settings/prod.json') as config_file:
        settings = json.load(config_file)

        if 'cmsCoreContext' not in settings:
            click.echo('Incomplete settings file.')
            exit(1)

        if 'cmsDbContext' not in settings:
            click.echo('Incomplete settings file.')
            exit(1)

        cms_core_context = FlaskContext(settings['cmsCoreContext'])
        cms_db_context = MongoDbContext(settings['cmsDbContext'])
        ctx.obj['elemental_context'] = ElementalContext(cms_core_context, cms_db_context)


@cli.command('version')
def version():
    click.echo(__version__)


@cli.command('init')
@click.pass_context
def init(ctx):

    if os.path.exists('.elemental'):
        click.echo('Elemental CMS has been already initialized.')
        return

    context: ElementalContext = ctx.obj['elemental_context']
    db_name = MongoDbConnectionManager.get_db_name(context.cms_db_context)
    if db_name is None:
        click.echo('The database context has something wrong.')
        return

    click.echo(f'Initializing Elemental CMS for db {db_name}...')

    CreateExpirationIndex(context.cms_db_context).execute()

    for path in ['media',
                 'static',
                 f'static/{context.cms_core_context.APP_NAME}',
                 'templates',
                 'translations',
                 'workspace',
                 'workspace/global_deps',
                 'workspace/snippets',
                 'workspace/pages']:
        create_folder(path)

    lib_root_path = pathlib.Path(__file__).resolve().parent
    path_parts = str(lib_root_path).split(os.sep)
    lib_path = os.sep.join(path_parts[1:-1])
    if not os.path.exists('templates/base.html'):
        copyfile(f'/{lib_path}/templates/base.html', 'templates/base.html')
    init_file = open('.elemental', mode="w", encoding="utf-8")
    init_file.write(json.dumps({
        'time': datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S')
    }, indent=4))
    init_file.close()
    click.echo('Initialization completed...')


def create_folder(path):
    if not os.path.exists(path):
        os.mkdir(path)


cli.add_command(GlobalDeps())
cli.add_command(Snippets())
cli.add_command(Pages())
cli.add_command(Static())
cli.add_command(Media())



from elementalcms.core import MongoDbContext
from elementalcms.persistence.repositories import GenericRepository


class DraftsRepository(GenericRepository):

    def __init__(self, db_context: MongoDbContext):
        super().__init__(db_context, 'drafts')



import os

import click
from google.cloud import storage
from google.cloud.storage import Bucket

from elementalcms.core import ElementalContext


class List:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, path):

        if not bool(self.context.cms_core_context.MEDIA_BUCKET
                    and not self.context.cms_core_context.MEDIA_BUCKET.isspace()):
            click.echo('MEDIA_BUCKET parameter not found on current settings.')
            return

        if path == '':
            click.echo('Empty string is not a valid folder, specify either / for root folder or '
                       'folder_name/ for any folder or sub-folder path.')
            return

        prefix = path
        delimiter = '/'

        if path is None:
            prefix = None
            delimiter = None
        elif not path[-1] == '/':
            click.echo('Remember to end your folder and/or sub-folder path with a /')
            return

        if path == '/':
            prefix = ''

        if self.context.cms_core_context.GOOGLE_SERVICE_ACCOUNT_INFO:
            client = storage.Client.from_service_account_info(self.context.cms_core_context.GOOGLE_SERVICE_ACCOUNT_INFO)
        else:
            client = storage.Client()

        bucket: Bucket = client.bucket(self.context.cms_core_context.MEDIA_BUCKET)
        objects = bucket.list_blobs(prefix=prefix, delimiter=delimiter)

        folders = set()
        remote_files = []

        for obj in objects:
            obj_name_parts = obj.name.split('/')
            folder_path = f'{"/".join(obj_name_parts[:-1])}/'
            folders.add(folder_path if folder_path == '/' else folder_path[:-1])
            if obj.name != folder_path:
                remote_files.append(obj.name)

        media_folder = self.context.cms_core_context.MEDIA_FOLDER
        local_files = []
        for root, directories, files in os.walk(media_folder):
            clean_root = root.replace(f'{media_folder}', '') or '/'
            clean_root = clean_root if clean_root == '/' else clean_root[1:]
            if clean_root not in folders:
                continue
            for file in files:
                local_files.append(os.path.join(root, file).replace(f'{media_folder}/', ''))
        all_files = set(local_files + remote_files)

        for file in sorted(all_files):
            if file in remote_files and file in local_files:
                click.echo(file)
            elif file in remote_files:
                click.echo(f'{file} * ')
            else:
                click.echo(f' * {file}')

        if len(all_files) == 0:
            if path:
                click.echo(f'\nNo media files found at {path}')
            else:
                click.echo(f'\nNo media files found')
            return
        if path:
            click.echo(f'\n{len(all_files)} media files found at {path}')
        else:
            click.echo(f'\n{len(all_files)} media files found')
        if local_files != remote_files:
            click.echo('* is an indicator of missing files either on remote folder (left) or local folder (right)')



from elementalcms.core import FlaskContext, MongoDbContext


class ElementalContext(object):

    def __init__(self,
                 cms_core_context: FlaskContext,
                 cms_db_context: MongoDbContext):

        self.cms_core_context: FlaskContext = cms_core_context
        self.cms_db_context: MongoDbContext = cms_db_context



from elementalcms.persistence.repositories import GlobalDepsRepository
from elementalcms.services import UseCaseResult, NoResult, Success
from elementalcms.core import MongoDbContext


class GetMe:
    __db_context: MongoDbContext

    def __init__(self, db_context: MongoDbContext):
        self.__db_context = db_context

    def execute(self, name, _type) -> UseCaseResult:
        repo = GlobalDepsRepository(self.__db_context)
        page = 0
        page_size = 100
        result = repo.find({'name': name, 'type': _type}, page=page, page_size=page_size)
        if result['total'] == 0:
            return NoResult()
        dep = result['items'][0]
        return Success(dep)



import click
from cloup import constraint, option, command, pass_context
from cloup.constraints import RequireExactly

from .pagescommands import Create, Remove, Push, Pull, List, Publish, Unpublish


class Pages(click.Group):

    def __init__(self):
        super(Pages, self).__init__()
        self.name = 'pages'
        self.add_command(self.list)
        self.add_command(self.create)
        self.add_command(self.remove)
        self.add_command(self.push)
        self.add_command(self.pull)
        self.add_command(self.publish)
        self.add_command(self.unpublish)

    @staticmethod
    @command(name='list',
             help='Pages list.')
    @option('--drafts',
            is_flag=True,
            help='List the pages whit a draft version (false by default).')
    @pass_context
    def list(ctx, drafts):
        List(ctx).exec(drafts)

    @staticmethod
    @command(name='create', help='Create a new page on your local workspace.')
    @option('--name',
            '-n',
            required=True,
            help='Page name. It must be unique, lowercased and it can not contains special characters.')
    @option('--lang',
            '-l',
            required=True,
            help=f'Page language. It must be specified even if the multi language mode is deactivated.')
    @pass_context
    def create(ctx, name, lang):
        Create(ctx).exec(name, lang)

    @staticmethod
    @command(name='remove', help='Remove an unpublished page. This command removes the page draft version; if you '
                                 'want to remove the page published version you must use the pages unpublish command.')
    @option('--name',
            '-n',
            required=True,
            help='Name of the page to be removed.')
    @option('--lang',
            '-l',
            required=True,
            help=f'Language version to be removed.')
    @pass_context
    def remove(ctx, name, lang):
        Remove(ctx).exec(name, lang)

    @staticmethod
    @command(name='push',
             help='Push page(s) specs and contents into the CMS database. All pushed pages have the draft collection '
                  'as its destination.')
    @option('--all',
            is_flag=True,
            help='Push all pages.')
    @option('--page',
            '-p',
            nargs=2,
            multiple=True,
            help='Name and language for the page to be pushed. For example: push -p home en -p home es')
    @constraint(RequireExactly(1), ['all', 'page'])
    @pass_context
    def push(ctx, **params):
        if params['all']:
            click.echo('Operation not ready yet.')
            return
        Push(ctx).exec(params['page'])

    @staticmethod
    @command(name='publish',
             help='Publish an especific page.')
    @option('--name',
            '-n',
            required=True,
            help='Page name.')
    @option('--lang',
            '-l',
            required=True,
            help='Page language.')
    @pass_context
    def publish(ctx, name, lang):
        Publish(ctx).exec(name, lang)

    @staticmethod
    @command(name='unpublish',
             help='Unpublish an especific page.')
    @option('--name',
            '-n',
            required=True,
            help='Page name.')
    @option('--lang',
            '-l',
            required=True,
            help='Page language.')
    @pass_context
    def unpublish(ctx, name, lang):
        Unpublish(ctx).exec(name, lang)

    @staticmethod
    @command(name='pull',
             help='Pull page(s) specs and contents from the CMS database.')
    @option('--all',
            is_flag=True,
            help='Pull all pages.')
    @option('--page',
            '-p',
            nargs=2,
            multiple=True,
            help='Name and language for the page to be pulled. For example: pull -p home en -p home es')
    @constraint(RequireExactly(1), ['all', 'page'])
    @option('--drafts',
            is_flag=True,
            help='Use this option to pull the pages draft version.')
    @pass_context
    def pull(ctx, **params):
        if params['all']:
            click.echo('Operation not ready yet.')
            return
        Pull(ctx).exec(params['page'], params['drafts'])



import time
import os
from shutil import copyfile

import click
from bson import json_util
from elementalcms.core import ElementalContext
from elementalcms.services import NoResult
from elementalcms.services.snippets import GetMe


class Pull:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, names):
        snippets_folder = self.context.cms_core_context.SNIPPETS_FOLDER
        for name in names:
            get_me_result = GetMe(self.context.cms_db_context).execute(name)
            if get_me_result.is_failure() and not isinstance(get_me_result, NoResult):
                click.echo('Something went wrong and it was not possible to perform the operation.')
                continue
            snippet = get_me_result.value()
            if snippet is None:
                click.echo(f'Snippet {name} does not exists.')
                continue
            self.build_local_backup(name)
            html_content = snippet['content']
            del snippet['content']
            if not os.path.exists(snippets_folder):
                os.makedirs(snippets_folder)
            spec_file_destination_path = f'{snippets_folder}/{name}.json'
            spec_file = open(spec_file_destination_path, mode="w", encoding="utf-8")
            spec_file.write(json_util.dumps(snippet, indent=4))
            spec_file.close()
            content_file_destination_path = f'{snippets_folder}/{name}.html'
            content_file = open(content_file_destination_path, mode="w", encoding="utf-8")
            content_file.write(html_content)
            content_file.close()
            click.echo(f'Snippet {name} pulled successfully from {self.context.cms_db_context.host_name}/{self.context.cms_db_context.database_name}.')

    def build_local_backup(self, name):
        click.echo('Building local backup...')
        snippets_folder = self.context.cms_core_context.SNIPPETS_FOLDER
        backups_folder_path = f'{snippets_folder}/.bak'
        if not os.path.exists(backups_folder_path):
            os.makedirs(backups_folder_path)
        sufix = round(time.time())
        local_spec_file_path = f'{snippets_folder}/{name}.json'
        local_content_file_path = f'{snippets_folder}/{name}.html'
        backup_spec_file_path = f'{snippets_folder}/.bak/{name}-local-{sufix}.json'
        backup_content_file_path = f'{snippets_folder}/.bak/{name}-local-{sufix}.html'
        if os.path.exists(local_spec_file_path):
            copyfile(local_spec_file_path, backup_spec_file_path)
        if os.path.exists(local_content_file_path):
            copyfile(local_content_file_path, backup_content_file_path)



import click

from elementalcms.core import ElementalContext
from elementalcms.services.global_deps import GetAll


class List:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self):
        result = GetAll(self.context.cms_db_context).execute()
        if result.is_failure():
            click.echo('There are no global dependencies to list. Create your first one using the '
                       '[global-deps create] command.')
            return

        for dep in result.value():
            click.echo(f'{dep["type"]} <-> {dep["name"]}')



import time
import os
from typing import Optional
import click
from bson import json_util

from elementalcms.core import ElementalContext
from elementalcms.services.global_deps import GetMe, RemoveOne


class Remove:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, name, _type) -> Optional[str]:
        get_me_result = GetMe(self.context.cms_db_context).execute(name, _type)
        if get_me_result.is_failure():
            click.echo(f'Global dependency {name} ({_type}) does not exist.')
            return None
        dep = get_me_result.value()
        spec_backup_file_path = self.build_backup(dep)
        RemoveOne(self.context.cms_db_context).execute(dep['_id'])
        click.echo(f'Global dependency {name} ({_type}) removed successfully.')
        return spec_backup_file_path

    def build_backup(self, dep) -> str:
        click.echo('Building backup...')
        root_folder_path = self.context.cms_core_context.GLOBAL_DEPS_FOLDER
        type_folder_name = dep['type'].replace('/', '_')
        suffix = round(time.time())
        backups_folder_path = f'{root_folder_path}/{type_folder_name}/.bak'
        if not os.path.exists(backups_folder_path):
            os.makedirs(backups_folder_path)
        spec_backup_file_path = f'{backups_folder_path}/{dep["name"]}-{suffix}.json'
        spec_backup_file = open(spec_backup_file_path, mode='w', encoding='utf-8')
        spec_backup_file.write(json_util.dumps(dep, indent=4))
        spec_backup_file.close()
        click.echo(f'{dep["name"]} backup built successfully.')
        return spec_backup_file_path



from .genericrepository import GenericRepository
from .globaldepsrepository import GlobalDepsRepository
from .draftsrepository import DraftsRepository
from .pagesrepository import PagesRepository
from .snippetsrepository import SnippetsRepository
from .sessionsrepository import SessionsRepository



import datetime
import os

import click
from bson import ObjectId, json_util

from elementalcms import ElementalContext


class Create:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, name, lang):
        if lang not in self.context.cms_core_context.LANGUAGES:
            click.echo(f'"{lang}" language is not supported')
            return
        destination_folder = self.context.cms_core_context.PAGES_FOLDER
        destination_path = f'{destination_folder}/{lang}'
        if not os.path.exists(destination_path):
            os.makedirs(destination_path)
        spec_file_destination_path = f'{destination_path}/{name}.json'
        content_file_destination_path = f'{destination_path}/{name}.html'
        if os.path.exists(spec_file_destination_path):
            click.echo(f'A page with the name "{name}" in "{lang}" language already exists.')
            return
        spec = {
            '_id': ObjectId(),
            'name': name,
            'language': lang,
            'title': f'{name} page',
            'description': '',
            'isHome': False,
            'cssDeps': [],
            'jsDeps': [],
            'createdAt': datetime.datetime.utcnow(),
            'lastModifiedAt': datetime.datetime.utcnow()
        }
        spec_file = open(spec_file_destination_path, mode="w", encoding="utf-8")
        spec_file.write(json_util.dumps(spec, indent=4))
        spec_file.close()
        content_file = open(content_file_destination_path, mode="w", encoding="utf-8")
        content_file.write('<div></div>')
        content_file.close()
        click.echo(f'{destination_path}/{name}.json|html files has been created successfully.')



import click

from elementalcms.core import ElementalContext
from elementalcms.services import NoResult
from elementalcms.services.snippets import GetAll


class List:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self):
        result = GetAll(self.context.cms_db_context).execute()
        if result.is_failure():
            if isinstance(result, NoResult):
                click.echo('No snippets yet, create your first snippet running the [snippets create] command.')
                return
            click.echo('Something went wrong and it was not possible to retreive the snippets information list.')
            return

        for spec in result.value():
            click.echo(f'{spec["name"]}')



import os
import time

import click
from bson import json_util

from elementalcms.core import ElementalContext
from elementalcms.services import NoResult
from elementalcms.services.pages import GetMe, RemoveOne, GetOne


class Remove:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, name, lang):

        get_page_result = GetMe(self.context.cms_db_context).execute(name, lang, False, False)
        if get_page_result.is_failure():
            if not isinstance(get_page_result, NoResult):
                click.echo('Something went wrong and it was not possible to complete the operation.')
                return

        page = get_page_result.value()
        if page is not None:
            click.echo(f'Page {name} in "{lang}" language has to be unpublished first, in order to be removed.')
            return

        get_draft_result = GetMe(self.context.cms_db_context).execute(name, lang, True, False)
        if get_draft_result.is_failure():
            if not isinstance(get_draft_result, NoResult):
                click.echo('Something went wrong and it was not possible to complete the operation.')
                return
        draft = get_draft_result.value()
        if draft is None:
            click.echo(f'Page {name} in "{lang}" language does not have a draft version.')
            return

        self.build_draft_backup(draft['_id'])

        RemoveOne(self.context.cms_db_context).execute(draft['_id'], True)
        click.echo(f'Page {name} in "{lang}" language removed successfully.')

    def build_draft_backup(self, _id):
        get_one_result = GetOne(self.context.cms_db_context).execute(_id, True)
        if get_one_result.is_failure():
            return
        click.echo('Building backup...')
        page = get_one_result.value()
        html_content = page['content']
        del page['content']
        backups_folder_path = f'{self.context.cms_core_context.PAGES_FOLDER}/{page["language"]}/.bak'
        if not os.path.exists(backups_folder_path):
            os.makedirs(backups_folder_path)
        sufix = round(time.time())
        spec_file_destination_path = f'{backups_folder_path}/{page["name"]}-draft-{sufix}.json'
        spec_file = open(spec_file_destination_path, mode="w", encoding="utf-8")
        spec_file.write(json_util.dumps(page, indent=4))
        spec_file.close()
        content_file_destination_path = f'{backups_folder_path}/{page["name"]}-draft-{sufix}.html'
        content_file = open(content_file_destination_path, mode="w", encoding="utf-8")
        content_file.write(html_content)
        content_file.close()



from dataclasses import dataclass

import pymongo.errors
from pymongo import MongoClient

from elementalcms.core import MongoDbContext


@dataclass
class MongoDbConnectionManager:

    __db_client: MongoClient = None

    @classmethod
    def _get_db(cls, context: MongoDbContext):
        if cls.__db_client is None:
            cls.__db_client = MongoClient(context.get_connection_string())
        return cls.__db_client.get_database(context.database_name)

    @classmethod
    def get_db_name(cls, context: MongoDbContext):
        try:
            db = cls._get_db(context)
            return db.name
        except pymongo.errors.ConnectionFailure:
            return None



import click
from cloup import constraint, option, command, pass_context
from cloup.constraints import RequireExactly

from .snippetscommands import Create, List, Push, Pull, Remove


class Snippets(click.Group):

    def __init__(self):
        super(Snippets, self).__init__()
        self.name = 'snippets'
        self.add_command(self.list)
        self.add_command(self.create)
        self.add_command(self.remove)
        self.add_command(self.push)
        self.add_command(self.pull)

    @staticmethod
    @command(name='list',
             help='Snippets list.')
    @pass_context
    def list(ctx):
        List(ctx).exec()

    @staticmethod
    @command(name='create', help='Create a new snippet on your local workspace.')
    @option('--name',
            '-n',
            required=True,
            help='Snippet name. It must be unique, lowercased and it can not contains special characters.')
    @pass_context
    def create(ctx, name):
        Create(ctx).exec(name)

    @staticmethod
    @command(name='push',
             help='Push snippet(s) specs into the CMS database.')
    @option('--all',
            is_flag=True,
            help='Push all snippets.')
    @option('--snippet',
            '-s',
            multiple=True,
            help='Name for the spec to be pushed. For example: push -s nav-bar')
    @constraint(RequireExactly(1), ['all', 'snippet'])
    @pass_context
    def push(ctx, **params):
        if params['all']:
            click.echo('Operation not ready yet.')
            return
        Push(ctx).exec(params['snippet'])

    @staticmethod
    @command(name='pull',
             help='Pull snippet(s) specs from the CMS database.')
    @option('--all',
            is_flag=True,
            help='Pull all snippets.')
    @option('--snippet',
            '-s',
            multiple=True,
            help='Name for the snippet to be pulled. For example: pull -d bootstrap application/javascript')
    @constraint(RequireExactly(1), ['all', 'snippet'])
    @pass_context
    def pull(ctx, **params):
        if params['all']:
            click.echo('Operation not ready yet.')
            return
        Pull(ctx).exec(params['snippet'])

    @staticmethod
    @command(name='remove', help='Remove an snippet.')
    @option('--name',
            '-n',
            required=True,
            help='Name of snippet to be removed.')
    @pass_context
    def remove(ctx, name):
        pass
        Remove(ctx).exec(name)



from datetime import datetime, timedelta
from uuid import uuid4

from flask.sessions import SessionInterface

from elementalcms.persistence.models import MongoSession
from elementalcms.services.sessions import GetMe, UpsertMe
from elementalcms.core import MongoDbContext


class MongoSessionInterface(SessionInterface):

    def __init__(self, db_context: MongoDbContext):
        self.db_context = db_context

    def open_session(self, app, request):
        sid = request.cookies.get(app.session_cookie_name)
        if sid:
            get_me_result = GetMe(self.db_context).execute(sid)
            if not get_me_result.is_failure():
                stored_session = get_me_result.value()
                if stored_session.get('expiration') > datetime.utcnow():
                    return MongoSession(initial=stored_session['data'],
                                        sid=stored_session['sid'])
        sid = str(uuid4())
        return MongoSession(sid=sid)

    def save_session(self, app, session: MongoSession, response):
        domain = self.get_cookie_domain(app)
        if not session:
            response.delete_cookie(app.session_cookie_name, domain=domain)
            return
        if self.get_expiration_time(app, session):
            expiration = self.get_expiration_time(app, session)
        else:
            expiration = datetime.utcnow() + timedelta(hours=1)
        UpsertMe(self.db_context).execute({
                                              'sid': session.sid,
                                              'data': session,
                                              'expiration': expiration
                                          })
        response.set_cookie(app.session_cookie_name, session.sid,
                            expires=self.get_expiration_time(app, session),
                            httponly=True, domain=domain)



import pycountry
from flask import render_template, Blueprint, request, g, current_app, redirect, abort, session, \
    render_template_string

from elementalcms.services import UseCaseResult
from elementalcms.services.pages import GetHome, GetMe


presenter = Blueprint('presenter', __name__, template_folder='templates')


@presenter.before_request
def before_request():
    lang_code = g.get('lang_code', None)
    if lang_code is None:
        abort(404)
    if lang_code in current_app.config['LANGUAGES']:
        return
    return redirect(request.full_path.replace(lang_code,
                                              session.get('lang_code', current_app.config["DEFAULT_LANGUAGE"])), 301)


@presenter.url_defaults
def add_language_code(endpoint, values):
    values.setdefault('lang_code', current_app.config["DEFAULT_LANGUAGE"])


@presenter.url_value_preprocessor
def pull_lang_code(endpoint, values):

    lang_code = values.get('lang_code', None)
    if lang_code is None:
        return

    country = pycountry.languages.get(alpha_2=lang_code)
    if country is None:
        return

    g.lang_code = lang_code


@presenter.route('/', methods=['GET'])
def index(lang_code: str):
    draft = request.args.get('draft')
    result: UseCaseResult = GetHome(current_app.config['CMS_DB_CONTEXT']).execute(lang_code,
                                                                                  draft=(draft == '1'))
    if result.is_failure():
        abort(404)
    session['lang_code'] = lang_code
    return render_template('presenter/index.html',
                           page=get_page_model(result.value()))


@presenter.route('/<slug>/', methods=['GET'])
def render(lang_code: str, slug: str):
    draft = request.args.get('draft')
    result: UseCaseResult = GetMe(current_app.config['CMS_DB_CONTEXT']).execute(slug,
                                                                                lang_code,
                                                                                draft=(draft == '1'))
    if result.is_failure():
        abort(404)
    session['lang_code'] = lang_code
    return render_template('presenter/index.html',
                           page=get_page_model(result.value()))


def get_page_model(page_spec):
    styles = get_styles(page_spec['cssDeps'])
    scripts = get_scripts(page_spec['jsDeps'])
    content = render_template_string(page_spec['content'], page=page_spec)
    return {
        'title': page_spec['title'],
        'name': page_spec['name'],
        'description': page_spec['description'],
        'content': content,
        'styles': ''.join(styles),
        'scripts': ''.join(scripts)
    }


def get_styles(deps):
    styles = []
    for dep in deps:
        if 'http' in dep['url']:
            styles.append(f'<link rel="stylesheet" type="text/css" href="{dep["url"]}">')
            continue
        href = f'{current_app.config["STATIC_URL"]}/{current_app.config["APP_NAME"]}/{dep["url"]}'
        styles.append(f'<link rel="stylesheet" type="text/css" href="{href}">')
    return styles


def get_scripts(deps):
    scripts = []
    for dep in deps:
        if dep['url'].startswith('http'):
            scripts.append(f'<script src="{dep["url"]}" type="{dep["type"]}"></script>')
            continue
        src = f'{current_app.config["STATIC_URL"]}/{current_app.config["APP_NAME"]}/{dep["url"]}'
        scripts.append(f'<script src="{src}" type="{dep["type"]}"></script>')
    return scripts



import time
import os

import click
from bson import json_util, ObjectId
from elementalcms.core import ElementalContext
from elementalcms.services.pages import GetOne, UpdateOne


class Push:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, names_tuples):
        for name_tuple in names_tuples:
            name = name_tuple[0]
            lang = name_tuple[1]
            if lang not in self.context.cms_core_context.LANGUAGES:
                click.echo(f'"{lang}" language not supported.')
                continue
            pages_folder_path = self.context.cms_core_context.PAGES_FOLDER
            spec_file_path = f'{pages_folder_path}/{lang}/{name}.json'
            content_file_path = f'{pages_folder_path}/{lang}/{name}.html'
            if not os.path.exists(content_file_path):
                click.echo(f'There is no content file for page {name} in {lang} language.')
                continue
            with open(spec_file_path) as page_spec_content:
                try:
                    page_spec = json_util.loads(page_spec_content.read())
                    if '_id' not in page_spec:
                        click.echo(f'Spec invalid for page {name} in {lang} language.')
                        continue
                    if not ObjectId.is_valid(page_spec['_id']):
                        click.echo(f'Spec invalid for page {name} in {lang} language.')
                        continue
                    if 'name' not in page_spec:
                        click.echo(f'Spec invalid for page {name} in {lang} language.')
                        continue
                    if page_spec['name'] != name:
                        click.echo(f'Spec invalid for page {name} in {lang} language.')
                        continue
                    with open(content_file_path) as page_content:
                        page_spec['content'] = page_content.read()
                        _id = page_spec['_id']
                        self.build_draft_backup(_id)
                        update_me_result = UpdateOne(self.context.cms_db_context).execute(_id, page_spec, True)
                        if update_me_result.is_failure():
                            click.echo(f'Something went wrong and it was not possible to push page {name} into {self.context.cms_db_context.host_name}/{self.context.cms_db_context.database_name}.')
                            continue
                        click.echo(f'Page {name} pushed successfully into {self.context.cms_db_context.host_name}/{self.context.cms_db_context.database_name}.')
                except Exception as e:
                    click.echo(e)
                    click.echo(f'Spec invalid for page {name} in {lang} language.')

    def build_draft_backup(self, _id):
        get_one_result = GetOne(self.context.cms_db_context).execute(_id, True)
        if get_one_result.is_failure():
            return
        click.echo('Building backup...')
        page = get_one_result.value()
        html_content = page['content']
        del page['content']
        backups_folder_path = f'{self.context.cms_core_context.PAGES_FOLDER}/{page["language"]}/.bak'
        if not os.path.exists(backups_folder_path):
            os.makedirs(backups_folder_path)
        sufix = round(time.time())
        spec_file_destination_path = f'{backups_folder_path}/{page["name"]}-draft-{sufix}.json'
        spec_file = open(spec_file_destination_path, mode="w", encoding="utf-8")
        spec_file.write(json_util.dumps(page, indent=4))
        spec_file.close()
        content_file_destination_path = f'{backups_folder_path}/{page["name"]}-draft-{sufix}.html'
        content_file = open(content_file_destination_path, mode="w", encoding="utf-8")
        content_file.write(html_content)
        content_file.close()



import os
import time

import click
from bson import json_util

from elementalcms.core import ElementalContext
from elementalcms.services import NoResult
from elementalcms.services.snippets import GetMe, RemoveOne, GetOne


class Remove:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, name):

        get_me_result = GetMe(self.context.cms_db_context).execute(name)
        if get_me_result.is_failure():
            if not isinstance(get_me_result, NoResult):
                click.echo('Something went wrong and it was not possible to complete the operation.')
                return
        snippet = get_me_result.value()
        if snippet is None:
            click.echo(f'Snippet {name} does not exists.')
            return

        self.build_backup(snippet['_id'])

        RemoveOne(self.context.cms_db_context).execute(snippet['_id'])
        click.echo(f'Snippet {name} removed successfully.')

    def build_backup(self, _id):
        get_one_result = GetOne(self.context.cms_db_context).execute(_id)
        if get_one_result.is_failure():
            return
        click.echo('Building backup...')
        snippet = get_one_result.value()
        html_content = snippet['content']
        del snippet['content']
        backups_folder_path = f'{self.context.cms_core_context.SNIPPETS_FOLDER}/.bak'
        if not os.path.exists(backups_folder_path):
            os.makedirs(backups_folder_path)
        sufix = round(time.time())
        spec_file_destination_path = f'{backups_folder_path}/{snippet["name"]}-{sufix}.json'
        spec_file = open(spec_file_destination_path, mode="w", encoding="utf-8")
        spec_file.write(json_util.dumps(snippet, indent=4))
        spec_file.close()
        content_file_destination_path = f'{backups_folder_path}/{snippet["name"]}-{sufix}.html'
        content_file = open(content_file_destination_path, mode="w", encoding="utf-8")
        content_file.write(html_content)
        content_file.close()



import time
import os
from typing import Optional

import click
from bson import json_util, ObjectId

from elementalcms.core import ElementalContext
from elementalcms.services.global_deps import GetOne, UpdateOne


class Push:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, deps) -> [str]:

        root_folder_path = self.context.cms_core_context.GLOBAL_DEPS_FOLDER

        if isinstance(deps, str):
            deps_tuples = []
            for r, d, f in os.walk(root_folder_path):
                for file in f:
                    deps_tuples.append((file.split('.')[0], r.split('/')[-1].replace('_', '/')))
            if len(deps_tuples) == 0:
                click.echo('There are no global dependencies to push.')
                return
        else:
            deps_tuples = deps

        spec_backup_file_paths = []
        for element in deps_tuples:
            name = element[0]
            _type = element[1]
            if _type not in ['application/javascript',
                             'text/css',
                             'module']:
                click.echo(f'"{_type}" type is not supported.')
                continue
            type_folder_name = _type.replace('/', '_')
            folder_path = f'{root_folder_path}/{type_folder_name}'
            spec_file_path = f'{folder_path}/{name}.json'
            if not os.path.exists(spec_file_path):
                click.echo(f'There is no spec file for {name} ({_type}).')
                continue
            with open(spec_file_path) as spec_file:
                try:
                    dep = json_util.loads(spec_file.read())
                except Exception as e:
                    click.echo(e)
                    click.echo(f'Invalid spec for dependency {name} ({_type}).')
                    continue
                if '_id' not in dep:
                    click.echo(f'Missing spec _id for: {name} ({_type})')
                    continue
                if not ObjectId.is_valid(dep['_id']):
                    click.echo(f'Invalid spec _id for: {name} ({_type})')
                    continue
                if 'name' not in dep:
                    click.echo(f'Missing spec name for: {name} ({_type})')
                    continue
                if dep['name'] != name:
                    click.echo(f'Invalid spec name for: {name} ({_type})')
                    continue
                _id = dep['_id']
                spec_backup_file_path = self.build_backup(_id)
                UpdateOne(self.context.cms_db_context).execute(_id, dep)
                click.echo(f'Global dependency {name} ({_type}) pushed successfully.')
                if spec_backup_file_path is not None:
                    spec_backup_file_paths.append(spec_backup_file_path)
        return spec_backup_file_paths

    def build_backup(self, _id) -> Optional[str]:
        get_one_result = GetOne(self.context.cms_db_context).execute(_id)
        if get_one_result.is_failure():
            return None
        click.echo('Building backup...')
        root_folder_path = self.context.cms_core_context.GLOBAL_DEPS_FOLDER
        dep = get_one_result.value()
        type_folder_name = dep['type'].replace('/', '_')
        folder_path = f'{root_folder_path}/{type_folder_name}'
        suffix = round(time.time())
        backups_folder_path = f'{folder_path}/.bak'
        if not os.path.exists(backups_folder_path):
            os.makedirs(backups_folder_path)
        spec_backup_file_path = f'{backups_folder_path}/{dep["name"]}-{suffix}.json'
        spec_backup_file = open(spec_backup_file_path, mode='w', encoding='utf-8')
        spec_backup_file.write(json_util.dumps(dep, indent=4))
        spec_backup_file.close()
        return spec_backup_file_path



import time
import os
from shutil import copyfile
from typing import Optional

import click
from bson import json_util
from google.cloud import storage
from google.cloud.storage import Bucket

from elementalcms.core import ElementalContext


class Pull:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, folder):

        if isinstance(folder, str):
            prefix = None
            delimiter = None
        else:
            prefix = folder[0]
            delimiter = '/'

        media_folder = self.context.cms_core_context.MEDIA_FOLDER

        if self.context.cms_core_context.GOOGLE_SERVICE_ACCOUNT_INFO:
            client = storage.Client.from_service_account_info(self.context.cms_core_context.GOOGLE_SERVICE_ACCOUNT_INFO)
        else:
            client = storage.Client()

        bucket: Bucket = client.bucket(self.context.cms_core_context.MEDIA_BUCKET)
        objects = bucket.list_blobs(prefix=prefix, delimiter=delimiter)
        remote_files = []

        for obj in objects:
            obj_name_parts = obj.name.split('/')
            folder_path = f'{"/".join(obj_name_parts[:-1])}/'
            if obj.name != folder_path:
                remote_files.append(obj.name)

        for file in remote_files:
            blob = bucket.blob(file)
            blob_folder = '/'.join(file.split('/')[0:-1])
            local_folder = f'{media_folder}/{blob_folder}/'
            if not os.path.exists(local_folder):
                os.makedirs(local_folder)
            click.echo(f'Downloading {file}')
            blob.download_to_filename(f'{media_folder}/{file}')
            click.echo(f'{file} downloaded successfuly.')



import datetime
from elementalcms.persistence.repositories import GlobalDepsRepository
from elementalcms.services import UseCaseResult, Success
from elementalcms.core import MongoDbContext


class UpdateOne:
    __db_context: MongoDbContext

    def __init__(self, db_context: MongoDbContext):
        self.__db_context = db_context

    def execute(self, _id, dep) -> UseCaseResult:
        repo = GlobalDepsRepository(self.__db_context)
        if '_id' in dep:
            del dep['_id']
        dep['lastModifiedAt'] = datetime.datetime.utcnow()
        success = repo.replace_one(_id, dep, True)
        return Success(success)



class FlaskContext(object):

    def __init__(self, settings: dict):
        self.DEBUG = settings.get('DEBUG', True)
        self.ENV = settings.get('ENV', 'development')
        self.SECRET = settings.get('SECRET', '2+2==10')
        self.SITE_NAME = settings.get('SITE_NAME', 'My Elemental CMS')
        self.GOOGLE_SITE_VERIFICATION_TOKEN = settings.get('GOOGLE_SITE_VERIFICATION_TOKEN', None)
        self.COMPANY = settings.get('COMPANY', 'Paranoid Software')
        self.CANONICAL_URL = settings.get('CANONICAL_URL', 'https://elemental.cms')
        self.LANGUAGES = settings.get('LANGUAGES', ['en', 'es'])
        self.DEFAULT_LANGUAGE = settings.get('DEFAULT_LANGUAGE', 'es')
        self.LANGUAGE_MODE = settings.get('LANGUAGE_MODE', 'multi')
        self.APP_NAME = settings.get('APP_NAME', 'app')
        self.STATIC_FOLDER = settings.get('STATIC_FOLDER', 'static').strip() or 'static'
        self.MEDIA_FOLDER = settings.get('MEDIA_FOLDER', 'media').strip() or 'media'
        self.STATIC_BUCKET = settings.get('STATIC_BUCKET', '').strip() or None
        self.MEDIA_BUCKET = settings.get('MEDIA_BUCKET', '').strip() or None
        self.STATIC_URL = settings.get('STATIC_URL', '/static') or '/static'
        self.MEDIA_URL = settings.get('MEDIA_URL', '/media') or '/media'
        self.GLOBAL_DEPS_FOLDER = settings.get('GLOBAL_DEPS_FOLDER', 'workspace/global_deps')
        self.PAGES_FOLDER = settings.get('PAGES_FOLDER', 'workspace/pages')
        self.SNIPPETS_FOLDER = settings.get('SNIPPETS_FOLDER', 'workspace/snippets')
        self.GOOGLE_SERVICE_ACCOUNT_INFO = settings.get('GOOGLE_SERVICE_ACCOUNT_INFO', {})



import glob
import os
import pathlib

import click
from google.cloud import storage

from elementalcms import ElementalContext


class Collect:

    def __init__(self, ctx):
        self.context: ElementalContext = ctx.obj['elemental_context']

    def exec(self, pattern, ignore_internals):

        static_folder = self.context.cms_core_context.STATIC_FOLDER

        if not pattern.startswith(static_folder):
            click.echo(f'We can only collect files located at the static folder which right now is {static_folder}')
            return

        if self.context.cms_core_context.STATIC_BUCKET is None:
            click.echo('STATIC_BUCKET parameter not found on current settings.')
            return

        if self.context.cms_core_context.GOOGLE_SERVICE_ACCOUNT_INFO:
            client = storage.Client.from_service_account_info(self.context.cms_core_context.GOOGLE_SERVICE_ACCOUNT_INFO)
        else:
            client = storage.Client()

        bucket = client.bucket(self.context.cms_core_context.STATIC_BUCKET)
        if not ignore_internals:
            self.collect_platform_files(bucket)
        self.collect_app_files(bucket, pattern)

        click.echo('Collect command excecuted successfully.')

    @staticmethod
    def collect_platform_files(bucket):
        lib_root_path = pathlib.Path(__file__).resolve().parent.parent.parent
        source_folder = f'{lib_root_path}/static'
        for r, d, f in os.walk(source_folder):
            for file in f:
                source_file_name = f'{r}/{file}'
                destination_blob_name = source_file_name.replace(source_folder, '', 1).lstrip('/')
                blob = bucket.blob(destination_blob_name)
                blob.cache_control = 'private, max-age=180'
                try:
                    blob.upload_from_filename(source_file_name)
                except Exception as e:
                    print(e)
                click.echo(f'Uploading {source_file_name} to {destination_blob_name}')

    def collect_app_files(self, bucket, pattern):
        files = glob.glob(pattern)
        if len(files) == 0:
            return
        source_folder = self.context.cms_core_context.STATIC_FOLDER
        for file in files:
            destination_blob_name = file.replace(source_folder, '', 1).lstrip('/')
            blob = bucket.blob(destination_blob_name)
            blob.cache_control = 'private, max-age=180'
            blob.upload_from_filename(file)
            click.echo(f'Uploading {file} to {destination_blob_name}')

    def collect_app_files_deprecated(self, bucket):
        source_folder = self.context.cms_core_context.STATIC_FOLDER
        for r, d, f in os.walk(source_folder):
            for file in f:
                source_file_name = f'{r}/{file}'
                destination_blob_name = source_file_name.replace(source_folder, '', 1).lstrip('/')
                blob = bucket.blob(destination_blob_name)
                blob.cache_control = 'private, max-age=180'
                blob.upload_from_filename(source_file_name)
                click.echo(f'Uploading {source_file_name} to {destination_blob_name}')



import os
import pathlib
from flask import Flask, request, send_from_directory, redirect, g, render_template_string
from flask_babel import Babel
from markupsafe import Markup

from elementalcms.core import ElementalContext

from elementalcms.admin.views import admin
from elementalcms.persistence import MongoSessionInterface
from elementalcms.presenter.views import presenter
from elementalcms.services.snippets import GetMe

__version__ = "1.0.82"


class Elemental:

    def __init__(self, app: Flask, context: ElementalContext):

        app.config.from_object(context.cms_core_context)

        # Passing DB Context to Blueprints
        app.config['CMS_DB_CONTEXT'] = context.cms_db_context

        translations_paths = [
            'translations',
            # Support for package translations
            f'{pathlib.Path(__file__).resolve().parent}/translations'
        ]
        app.config['BABEL_TRANSLATION_DIRECTORIES'] = ';'.join(translations_paths)

        admin.url_prefix = None if context.cms_core_context.LANGUAGE_MODE == 'single' else '/<lang_code>'
        app.register_blueprint(admin)
        presenter.url_prefix = None if context.cms_core_context.LANGUAGE_MODE == 'single' else '/<lang_code>'
        app.register_blueprint(presenter)

        # Session support using MongoDB
        app.session_interface = MongoSessionInterface(context.cms_db_context)

        babel = Babel(app)

        @babel.localeselector
        def get_locale():
            if not g.get('lang_code', None):
                g.lang_code = request.accept_languages.best_match(context.cms_core_context.LANGUAGES)
            return g.lang_code

        @app.before_request
        def before_request():
            path = request.full_path

            if 'media' in path:
                if context.cms_core_context.DEBUG:
                    path_parts = path.strip('?').split('/')
                    folders = '/'.join(path_parts[0:len(path_parts) - 1])
                    local_path = os.path.join(app.root_path, folders.lstrip('/'))
                    return send_from_directory(local_path, path_parts[-1])
                return redirect(f'{context.cms_core_context.MEDIA_URL}{path.replace("/media", "")}')

            if 'static' in path:
                if context.cms_core_context.DEBUG and ('static/admin' in path or 'static/presenter' in path):
                    path_parts = path.strip('?').split('/')
                    local_path = pathlib.Path(__file__).resolve().parent
                    return send_from_directory(local_path, '/'.join(path_parts[1:]))
                return

            if request.full_path == '/?':
                if context.cms_core_context.LANGUAGE_MODE == 'single':
                    return
                return redirect(f'/{context.cms_core_context.DEFAULT_LANGUAGE}')

        @app.context_processor
        def elemental_static_url_for_processor():
            def elemental_static_url_for(file_path=''):
                return f'{context.cms_core_context.STATIC_URL}/{file_path}'
            return dict(elemental_static_url_for=elemental_static_url_for)

        @app.context_processor
        def elemental_media_url_for_processor():
            def elemental_media_url_for(file_path=''):
                return f'{context.cms_core_context.MEDIA_URL}/{file_path}'
            return dict(elemental_media_url_for=elemental_media_url_for)

        @app.context_processor
        def render_snippet_processor():
            def render_snippet(name):
                get_me_result = GetMe(context.cms_db_context).execute(name)
                if get_me_result.is_failure():
                    raise Exception(f'There are no snippets under the name {name}')
                content = render_template_string(get_me_result.value()['content'])
                return Markup(content)
            return dict(render_snippet=render_snippet)



from elementalcms.persistence.repositories import GlobalDepsRepository, PagesRepository, DraftsRepository
from elementalcms.services import UseCaseResult, NoResult, Success
from elementalcms.core import MongoDbContext


class GetHome:
    __db_context: MongoDbContext

    def __init__(self, db_context: MongoDbContext):
        self.__db_context = db_context

    def execute(self, language, draft=False, add_gloabl_deps=True) -> UseCaseResult:
        if draft:
            repo = DraftsRepository(self.__db_context)
        else:
            repo = PagesRepository(self.__db_context)
        result = repo.find({'isHome': True, 'language': language}, page=0, page_size=10)
        if result['total'] == 0:
            return NoResult()
        page = result['items'][0]
        if add_gloabl_deps:
            global_deps = self.get_all_global_deps()
            css_deps = [d for d in global_deps if d['type'] == 'text/css']
            page['cssDeps'] = css_deps + page['cssDeps']
            js_deps = [d for d in global_deps if d['type'] == 'application/javascript']
            page['jsDeps'] = js_deps + page['jsDeps']
        return Success(page)

    def get_all_global_deps(self):
        repo = GlobalDepsRepository(self.__db_context)
        all_deps = []
        page = 0
        page_size = 50
        while True:
            result = repo.find(sort={'type': 1, 'order': 1}, page=page, page_size=page_size)
            total = result['total']
            if total == 0:
                return []
            all_deps.extend(result['items'])
            if len(all_deps) >= total:
                break
            page += 1
        return all_deps



from bson import ObjectId

from elementalcms.persistence import MongoDbConnectionManager
from elementalcms.core import MongoDbContext


class GenericRepository(MongoDbConnectionManager):

    def __init__(self, db_context: MongoDbContext, coll_name: str):
        self._coll = self._get_db(db_context).get_collection(coll_name)

    def find_one(self, _id):
        if not ObjectId.is_valid(_id):
            return None
        find_one_result = self._coll.find_one({'_id': ObjectId(_id)})
        return find_one_result

    def delete_one(self, _id) -> bool:
        if not ObjectId.is_valid(_id):
            return False
        delete_one_result = self._coll.delete_one({'_id': ObjectId(_id)})
        return delete_one_result.deleted_count == 1

    def insert_one(self, payload) -> str:
        insert_result = self._coll.insert_one(payload)
        return str(insert_result.inserted_id)

    def replace_one(self, _id, spec, upsert=True) -> bool:
        if not ObjectId.is_valid(_id):
            return False
        update_result = self._coll.replace_one({'_id': ObjectId(_id)}, spec, upsert=upsert)
        return update_result.raw_result['n'] == 1 or update_result.modified_count == 1

    def find(self, query=None, sort=None, page=0, page_size=10):

        result = {
            'items': [],
            'total': 0
        }

        data = [
                    {'$match': query if query else {}},
                    {'$skip': page * page_size},
                    {'$limit': page_size}
                ]
        total = [
                    {'$match': query if query else {}},
                    {'$count': 'total'}
                ]

        if sort:
            data.append({
                '$sort': sort
            })
            total.append({
                '$sort': sort
            })

        cursor = self._coll.aggregate([{
            '$facet': {
                'data': data,
                'total': total
            }
        }])

        aggregate_result = list(cursor)
        if len(aggregate_result) == 0:
            return result

        response = aggregate_result[0]
        for doc in response['data']:
            result['items'].append(doc)

        result['total'] = 0 if len(response['total']) == 0 else response['total'][0]['total']

        return result
